@@ -1,5 +1,5 @@
 """Models package for the models implemented in tiatoolbox."""
-from tiatoolbox.models import abc, architecture, dataset
+from tiatoolbox.models import architecture, dataset, engine, models_abc
 from tiatoolbox.models.engine.nucleus_instance_segmentor import NucleusInstanceSegmentor
 from tiatoolbox.models.engine.patch_predictor import (
     IOPatchPredictorConfig,
