@@ -1,6 +1,9 @@
 """This package contains various tools for working with WSIs."""
 from tiatoolbox.tools import (
+    graph,
     patchextraction,
+    pyramid,
+    registration,
     stainaugment,
     stainextract,
     stainnorm,
